/**
 * @author      ETSI / STF462 / Alexandre Berge
 * @version     $URL: svn+ssh://vcs.etsi.org/TTCN3/TOOL/ITS_TestPlatform/branches/STF462/javasrc/codec/org/etsi/ttcn/codec/generic/Union.java $
 *              $Id: Union.java 621 2013-09-18 13:51:09Z berge $
 */
package org.etsi.ttcn.codec.generic;

import org.etsi.ttcn.tci.Type;
import org.etsi.ttcn.tci.Value;
import org.etsi.ttcn.tci.UnionValue;
import org.etsi.ttcn.codec.ICodec;
import org.etsi.ttcn.codec.MainCodec;
import org.etsi.ttcn.codec.CodecBuffer;

public class Union extends ICodec {

    public Union(MainCodec mainCodec) {
        super(mainCodec);
    }

    @Override
    public Value decode(CodecBuffer buf, Type decodingHypothesis) {

        UnionValue uv = (UnionValue)decodingHypothesis.newInstance();
        String variant = "";

        System.out.println("Union.decode: looking for variant hint " + decodingHypothesis.getName());
        variant = mainCodec.getHint(decodingHypothesis.getName());
        
        if(variant != null) {
            Value value = uv.getVariant(variant);
            if (value == null) {
                value = mainCodec.getTciCDRequired().getUnionValue(decodingHypothesis, variant);
                if (value != null) {
                    uv.setVariant(variant, mainCodec.decode(buf, value.getType()));
                    return uv;
                }
            } else {
                uv.setVariant(variant, mainCodec.decode(buf, value.getType()));
                return uv;
            }
        }

        // no hint. Try all variants...
        String[] variants = uv.getVariantNames();
        for(int i=0; i < variants.length; i++) {
            // Copy buf to preserve it in case of failure
            CodecBuffer variantBuf = buf.getBuffer(0, buf.getNbBits());

            // Try to decode this variant
            Value v = uv.getVariant(variants[i]);
            if (v == null) {
                v = uv.getVariant(mainCodec.getHint(variants[i]));
            }
            Value vv = mainCodec.decode(buf, v.getType());
            if(vv != null) {
                buf.overwriteWith(variantBuf); 
                uv.setVariant(variants[i], vv);
                return uv;
            }
        }
        return null;
    }

    @Override
    public CodecBuffer encode(Value value) {

        UnionValue uv = (UnionValue)value;

        CodecBuffer buf = new CodecBuffer();
        preEncode(buf, uv);
        CodecBuffer variantBuf = mainCodec.encode(uv.getVariant(uv.getPresentVariantName()));
        buf.append(variantBuf);

        return buf;
    }

    protected void preEncode(CodecBuffer buf, UnionValue uv) {

    }
}

