/**
 * @author      ETSI / STF462 / Alexandre Berge
 * @version     $URL: https://forge.etsi.org/svn/ITS/branches/STF462/javasrc/codec/org/etsi/ttcn/codec/its/uppertester/UtPduId.java $
 *              $Id: UtPduId.java 1235 2013-10-17 08:56:30Z garciay $
 */
package org.etsi.ttcn.codec.its.uppertester;

import java.util.HashMap;
import java.util.Map;

public enum UtPduId {
    
    /* From LibItsCommon_TypesAndValues */
    UtInitialize(0x00),
    UtInitializeResult(0x01),
    UtChangePosition(0x02),
    UtChangePositionResult(0x03),
    UtResult(0x24), // FIXME: obsolete
    
    /* From LibItsDenm_TypesAndValues */
    UtDenmTrigger(0x10),
    UtDenmTriggerResult(0x11),
    UtDenmUpdate(0x12),
    UtDenmUpdateResult(0x13),
    UtDenmTermination(0x14),
    UtDenmTerminationResult(0x15),
    //reserved(0x16),
    UtDenmEventInd(0x17),
    
    /* From LibItsCam_TypesAndValues */
    //reserved(0x20),
    UtCamTriggerResult(0x21),
    //reserved(0x22),
    UtCamEventInd(0x23),
    UtCamTrigger_changeCurvature(0x30),
    UtCamTrigger_changeSpeed(0x31),
    UtCamTrigger_setAccelerationControlStatus(0x32),
    UtCamTrigger_setExteriorLightsStatus(0x33),
    UtCamTrigger_changeHeading(0x34),
    UtCamTrigger_setDriveDirection(0x35),
    UtCamTrigger_changeYawRate(0x36),
    UtCamTrigger_changePosition(0x37),// FIXME: obsolete
    UtCamTrigger_checkLdm(0x38),// FIXME: obsolete
    UtCamTrigger_setStationType(0x39),
    UtCamTrigger_setVehicleRole(0x3a),
    UtCamTrigger_setEmbarkationStatus(0x3b),
    UtCamTrigger_setPtActivation(0x3c),
    UtCamTrigger_setDangerousGoods(0x3d),
    UtCamTrigger_setDangerousGoodsExt(0x3e),
    UtCamTrigger_setLightBarSireneStatus(0x3f),
    
    /* From LibItsGeoNetworking_TypesAndValues */
    //reserved(0x40),
    UtGnTriggerResult(0x41),
    //reserved(0x42),
    UtGnEventInd(0x43),
    UtGnTrigger_geoUnicast(0x50),
    UtGnTrigger_geoBroadcast(0x51),
    UtGnTrigger_geoAnycast(0x52),
    UtGnTrigger_shb(0x53),
    UtGnTrigger_tsb(0x54),
    
    /* From LibItsBtp_TypesAndValues */
    //reserved(0x60),
    UtBtpTriggerResult(0x61),
    //reserved(0x62),
    UtBtpEventInd(0x63),
    UtBtpTrigger_A(0x70),
    UtBtpTrigger_B(0x71),
    
    /* From LibItsIpv6OverGeoNetworking_TypesAndValues */
    UtGn6Trigger(0x80),
    UtGn6TriggerResult(0x81),
    //reserved(0x82),
    UtGn6EventInd(0x83),
    UtGn6GetInterfaceInfo(0x84),
    UtGn6GetInterfaceInfoResult(0x85),
    
    /* From LibItsFntp_TypesAndValues */
    UtInitialize_utFntpInitialize(0x90),
    UtFntpEvent_nfSapPrimitivesDown(0x91),
    UtFntpEventInd_nfSapPrimitivesUp(0x9A),
    
    /* From LibItsFsap_TypesAndValues */
    UtInitialize_utFsapInitialize(0xa0),
    UtFsapEvent_faSapPrimitivesDown(0xa1),
    UtFsapEventInd_faSapPrimitivesUp(0xa2),
    
    /* From LibItsIicp_TypesAndValues */
    UtInitialize_utIicpInitialize(0xb0),
    UtIicpEvent_mcmdRs(0xb1),
    UtIicpEvent_mf_Request_request(0xb2),
    UtIicpEvent_mn_Request_request(0xb3),
    UtIicpEvent_mi_Request_request(0xb4),
    UtIicpEvent_mi_Get_request(0xb5),
    UtIicpEvent_mi_Set_request(0xb6),
    UtIicpEvent_mf_Command_confirm(0xb7),
    UtIicpEvent_mn_Command_confirm(0xb8),
    UtIicpEvent_mi_Command_confirm(0xb9),
    UtIicpEvent_mi_Get_confirm(0xba),
    UtIicpEvent_mi_Set_confirm(0xbb),
    UtIicpEventInd_mcmdRq(0xc0),
    UtIicpEventInd_mf_Command_request(0xc1),
    UtIicpEventInd_mn_Command_request(0xc2),
    UtIicpEventInd_mi_Command_request(0xc3),
    
    /* From LibItsAtsp_TypesAndValues */
    UtInitialize_utAtspInitialize(0xd0),
    UtAtspEvent_inSapPrimitivesDown(0xd1),
    UtAtspEventInd_inSapPrimitivesUp(0xd2),
    
    /* Reserved */
    reserved(0xFF);
    
    private byte value;
    private static final Map<String, Byte> UtPduIds = new HashMap<String, Byte>();
    private static final Map<Byte, String> UtPduNames = new HashMap<Byte, String>();
    
    private UtPduId(int value) {
        this.value = (byte)value;
    }
    
    public static Byte value(String name) {
        return UtPduIds.get(name);
    }
    
    public static String name(Byte value) {
        return UtPduNames.get(value);
    }
    
    static {
        for (UtPduId item : UtPduId.values()) {
            UtPduIds.put(item.name(), new Byte(item.value));
            UtPduNames.put(new Byte(item.value), item.name());
        }
    }
}