/**
 *  Layer factory (Singleton)
 *  
 *  @author     ETSI / STF424
 *  @version    $URL: svn+ssh://vcs.etsi.org/TTCN3/TOOL/ITS_TestPlatform/trunk/javasrc/adapter/org/etsi/its/adapter/layers/LayerFactory.java $
 *              $Id: LayerFactory.java 182 2011-04-27 08:01:07Z berge $
 *
 */
package org.etsi.its.configtester;


import java.lang.reflect.Constructor;
import java.util.Map;
import java.util.TreeMap;

/**
 *  Layer factory (Singleton)
 */
public class ConfigTesterFactory {

    /**
     * Unique instance of the factory
     */
    private static ConfigTesterFactory instance = new ConfigTesterFactory();
    
    /**
     * List of the registered layer types 
     */
    protected Map<String, Class<? extends IConfigTester>> _configTesters = new TreeMap<String, Class<? extends IConfigTester>>();
    protected Map<String, IConfigTester> _configTesterInstances = new TreeMap<String, IConfigTester>();
    
    /**
     * Private constructor (Singleton pattern)
     */
    private ConfigTesterFactory() {
        
        // Register the configTester
        _configTesters.put("GenericFntp", org.etsi.its.configtester.generic.FntpConfigTester.class);
        _configTesters.put("GenericFsap", org.etsi.its.configtester.generic.FsapConfigTester.class);
    }

    /**
     * Gets the unique factory instance
     * @return LayerFactory instance
     */
    public static ConfigTesterFactory getInstance(){
        return instance;
    }

    /**
     * Retrieves an config tester instance of the desired type
     * @param  cfName    Name of the config tester
     * @return Config tester instance
     */
    public IConfigTester getConfigTester(String cfName) {
        IConfigTester configTester = _configTesterInstances.get(cfName);
        if(configTester == null) {
        	configTester = createConfigTester(cfName);
            _configTesterInstances.put(cfName, configTester);
        }
        return configTester;
    }
    
    /**
     * Creates an config tester of the desired type
     * @param  cfName    Name of the config tester
     * @return Config tester instance
     */
    private IConfigTester createConfigTester(String cfName) {
        IConfigTester configTester = null;
        Class<?>[] ctorParams = {};
        
           try {        
            Class<? extends IConfigTester> cls = _configTesters.get(cfName);
            if (cls == null) {
                throw new RuntimeException("No class registered under " + cfName);
            }
            Constructor<? extends IConfigTester> ctor = cls.getConstructor(ctorParams);
            configTester = ctor.newInstance();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return configTester;
    }

    public void dispose() {
        for (String key : _configTesterInstances.keySet()) {
            _configTesterInstances.get(key).uninitialize();
        } // End of 'for' statement
    }
}
