/*
 * arguments
 * --debug --Ut=127.0.0.1:10000:10001 --Lt=127.0.0.1:10100:10101 --Ct=127.0.0.1:10200:10201 --Iicp=192.168.178.4:10300:10301
 */
package org.etsi.its.adapter.stf455.dispatcher;


import java.io.IOException;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.etsi.common.ByteHelper;
import org.etsi.its.adapter.stf455.dispatcher.CmdLineParser.IllegalOptionValueException;
import org.etsi.its.adapter.stf455.dispatcher.CmdLineParser.UnknownOptionException;


public class Dispatcher {
    
    /**
     * Module version
     */
    public static final String Version = "1.0.3.0";
    
    /**
     * Logger instance
     */
    //private final static Logger _logger = Logger.getLogger(Dispatcher.class.getName());
    private final static Logger _logger = Logger.getLogger("org.etsi.its.adapter.stf455.Dispatcher");
    
    /**
     * Debug mode. Default: false
     */
    private static Level Debug = Level.OFF;
    
    /**
     * UT module. Default: 0.0.0.0:0:0
     */
    private static String UtModule = "UT:0.0.0.0:13010:13110";
    
    /**
     * LT module. Default: 0.0.0.0:0:0
     */
    private static String LtModule = "LT:0.0.0.0:13011:13111";
    
    /**
     * CF module. Default: 0.0.0.0:0:0
     */
    private static String CfModule = "CF:0.0.0.0:13012:13112";
    
    /**
     * IICP module. Default: 0.0.0.0:0:0
     */
    private static String IicpModule = "IICP:0.0.0.0:0:0";
    
    /**
     * IUT ITS-SCU-ID. Default: 12
     */
    private static int SutScuIdParam = 12;
    
    /**
     * @param args
     */
    public static void main(String[] args) {
//        System.out.println(System.getProperty("java.library.path"));
//        System.setProperty("java.library.path", "C:\\windows\\system32");
        // Process command line arguments
        processCommandLine(args);
        // Set traces
        _logger.addHandler(new ConsoleHandler());
        _logger.setLevel(Debug);
        _logger.getHandlers()[0].setLevel(Debug);
        //Logger.getLogger("org.etsi.its.adapter.stf455.Dispatcher").addHandler(new ConsoleHandler());
        //Logger.getLogger("org.etsi.its.adapter.stf455.Dispatcher").setLevel(Debug);
        //Logger.getLogger("org.etsi.its.adapter.stf455.Dispatcher").getHandlers()[0].setLevel(Debug);
        //
        DispatcherTrigger dispatcher = new DispatcherTrigger(UtModule, LtModule, CfModule, IicpModule, SutScuIdParam);
        _logger.setLevel(Level.WARNING);	// prevent multiple logger output
        _logger.getHandlers()[0].setLevel(Level.WARNING);
        dispatcher.start();
        
        boolean bContinue = true;
        while (bContinue) {
            try {
                // Check keyboard
                if (System.in.available() > 0) {
                    int token = System.in.read();
                    switch (token)
                    {
                        case 'q':
                            bContinue = false;
                            _logger.info("Terminate application...");
                            break;
                        case '0':
                        	dispatcher.initDebugIicp();
                        	break;
                        case '1':	
                            byte[] m = new byte[] {(byte)0x00, (byte)0x0A, (byte)0xFF, (byte)0xFF, (byte)0x00, (byte)0x02,
                            		(byte)0x00, (byte)0x09, 
                            		(byte)0x05, (byte)0x01, (byte)0x03, 
                            		(byte)0x00, (byte)0x00, (byte)0x00,  
                            		(byte)0x00, (byte)0x01, (byte)0x00};
                            //iicpSim.DispatchMessage(m);
                            ByteHelper.dump("injecting UT msg:", m);
                            dispatcher.InjectMsgLtPort(m);
                        	break;
                        case '2':
                            byte[] n = new byte[] {(byte)0x00, (byte)0x0C, (byte)0x00, (byte)0x03, (byte)0x00, (byte)0x00,
                            		(byte)0x00, (byte)0xF1, 
                            		(byte)0x07, (byte)0x14, (byte)0x05, 
                            		(byte)0x01, (byte)0x03, (byte)0x00, (byte)0x00, 
                            		(byte)0x00,
                            		(byte)0x00, (byte)0x01, (byte)0x00};
                            //iicpSim.DispatchMessage(m);
                            ByteHelper.dump("injecting msg:", n);
                            dispatcher.DispatchMessage(n);
                        	break;
                        case '3':
                            byte[] m1 = new byte[] {(byte)0x00, (byte)0x0C, (byte)0x00, (byte)0x03, (byte)0x00, (byte)0x03,
                            		(byte)0x01, (byte)0x09, 
                            		(byte)0x03, (byte)0xFF, (byte)0x01, 
                            		(byte)0x00, (byte)0x00, 
                            		(byte)0x00, (byte)0x01, (byte)0x00};
                            //iicpSim.DispatchMessage(m);
                            ByteHelper.dump("injecting msg:", m1);
                            dispatcher.DispatchMessage(m1);
                        	break;
                        case '4':
                            byte[] m2 = new byte[] {(byte)0x00, (byte)0x0C, (byte)0x00, (byte)0x03, (byte)0x00, (byte)0x01, 
                        			(byte)0x00, (byte)0x0C, (byte)0x18, (byte)0x17, (byte)0x01, (byte)0x15, (byte)0x01,
                            		(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, 
                            		(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, 
                            		(byte)0x00, (byte)0x00, (byte)0x00, 
                            		(byte)0x08, (byte)0x00, (byte)0x00, 
                            		(byte)0x00, (byte)0x01, (byte)0x00
                            		};
                            //iicpSim.DispatchMessage(m);
                            ByteHelper.dump("injecting msg:", m2);
                            dispatcher.DispatchMessage(m2);
                        	break;
                        case '5':
                            byte[] m3 = new byte[] {(byte)0x00, (byte)0x0C, (byte)0x00, (byte)0x03, (byte)0x00, (byte)0x01,
                            		(byte)0x01, (byte)0x09, 
                            		(byte)0x02, (byte)0x01, (byte)0x15, 
                            		(byte)0x00,  
                            		(byte)0x00, (byte)0x01, (byte)0x00};
                            //iicpSim.DispatchMessage(m);
                            ByteHelper.dump("injecting msg:", m3);
                            dispatcher.DispatchMessage(m3);
                        	break;
                        case '6':
                            byte[] m4 = new byte[] {(byte)0xB1, (byte)0x01, (byte)0x15, (byte)0x01, 
                            		(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
                            		(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
                            		(byte)0x00, (byte)0x00, (byte)0x00, 
                            		(byte)0x08};
                            ByteHelper.dump("injecting UT msg:", m4);
                            dispatcher.InjectMsgUtPort(m4);
                        	break;
                        case '7':
                            byte[] m5 = new byte[] {(byte)0x00, (byte)0x0C, (byte)0x00, (byte)0x03, (byte)0x00, (byte)0x00,
                            		(byte)0x00, (byte)0x09, 
                            		(byte)0x17, (byte)0x01, 
                            		(byte)0x15, (byte)0x01,
                            		(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, 
                            		(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, 
                            		(byte)0x00, (byte)0x00, (byte)0x00, 
                            		(byte)0x08,
                            		(byte)0x00, (byte)0x01, (byte)0x00};
                            //iicpSim.DispatchMessage(m);
                            ByteHelper.dump("injecting msg:", m5);
                            dispatcher.DispatchMessage(m5);
                        	break;
                    }
                }
                // Wait
                java.lang.Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        } // End of 'while' statement
        dispatcher.terminate();
        System.exit(0);
    }

    /**
     * Parse the command line and set options
     * @param args Command line arguments
     */
    private static void processCommandLine(String[] args) {
        // Sanity checks
        if (args.length == 0) {
            return;
        }
        
        CmdLineParser parser = new CmdLineParser();
        CmdLineParser.Option debug = parser.addBooleanOption("debug");
        CmdLineParser.Option ut = parser.addStringOption("Ut");
        CmdLineParser.Option lt = parser.addStringOption("Lt");
        CmdLineParser.Option cf = parser.addStringOption("Cf");
        CmdLineParser.Option iicp = parser.addStringOption("Iicp");
        CmdLineParser.Option sut = parser.addStringOption("SutItsScuId");
        try {
            parser.parse(args);
        } catch (IllegalOptionValueException e) {
            e.printStackTrace();
            printUsage();
            System.exit(2);
        } catch (UnknownOptionException e) {
            e.printStackTrace();
            printUsage();
            System.exit(3);
        }
        
        Object value;
        if ((value = parser.getOptionValue(debug, false)) != null) {
            Debug = (((Boolean)value).booleanValue() == true) ? Level.ALL : Level.OFF;
        }
        if ((value = parser.getOptionValue(ut, "")) != null) {
            if (!((String)value).isEmpty()) {
                UtModule = String.format("UT:%s", (String)value);
            }
        }
        if ((value = parser.getOptionValue(lt, "")) != null) {
            if (!((String)value).isEmpty()) {
                LtModule = String.format("LT:%s", (String)value);
            }
        }
        if ((value = parser.getOptionValue(cf, "")) != null) {
            if (!((String)value).isEmpty()) {
                CfModule = String.format("CF:%s", (String)value);
            }
        }
        if ((value = parser.getOptionValue(iicp, "")) != null) {
            if (!((String)value).isEmpty()) {
                IicpModule = String.format("IICP:%s", (String)value);
            }
        }
        if ((value = parser.getOptionValue(sut, "12")) != null) {
            if (!((String)value).isEmpty()) {
                SutScuIdParam = Integer.parseInt(value.toString());
            }
        }
        
    } // End of method processCommandLine
    
    /**
     * Display application usage
     */
    private static void printUsage() {
        System.err.println("Usage: [--debug (default:false)]\n" +
                " [--Ut=192.168.1.2:13010:13110 \"UpperTester module - IPv4 address/remote ports/local port\" (default:0.0.0.0:0:0, not used)]\n" + 
                " [--Lt=192.168.1.2:13011:13111 \"LowerTester module - IPv4 address/remote ports/local port\" (default:0.0.0.0:0:0, not used)]\n" + 
                " [--Cf=192.168.1.2:13012:13112 \"Configuration module - IPv4 address/remote ports/local port\" (default:0.0.0.0:0:0, not used)]\n" + 
                " [--Iicp=192.168.1.2:13010:13110 \"SUT IICP access parameters - IPv4 address/remote ports/local port\" (default:0.0.0.0:0:0, not used)]\n" + 
                " [--SutItsScuId=12 \"SUT ITS-SCU-ID\" ]\n"
        );
    }
    
} // End of class Dispatcher
